<?php

namespace Drupal\drimage\Controller;

use Drupal\Core\Entity\EntityStorageException;
use Drupal\file\Entity\File;
use Drupal\image\Controller\ImageStyleDownloadController;
use Drupal\image\Entity\ImageStyle;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

/**
 * Simple extension over the default image download controller.
 *
 * We inherit from it so we have all functions and logic available. We just
 * override the way the image is generated to suit the needs of the dynamically
 * generated image styles.
 *
 * Images are scaled by default but cropping can be activated on the formatter
 * settings form.
 * When cropping is not activated a height of 0 is passed to the Controller.
 */
class DrImageController extends ImageStyleDownloadController {

  /**
   * @param $size array Dimensions array (0,1 for width,height)
   * @param $resize int New size for $dimension
   * @param $dimension int (0/1) which dimension to set to $resize.
   * @return mixed array New size array.
   */
  public function resize($size, $resize, $dimension) {
    $new = $size;

    if (!isset($size[$dimension]) || $size[$dimension] == 0) {
      return $new;
    }
    $new[$dimension] = $resize;

    $inverse_dimension = abs($dimension - 1);
    if (!isset($size[$inverse_dimension]) || $size[$inverse_dimension] == 0) {
      return $new;
    }

    $new[$inverse_dimension] = round($new[$inverse_dimension] * ($new[$dimension] / $size[$dimension]));
    return $new;
  }

  /**
   * Given a raw width and height: check if it adheres to the settings.
   *
   * @param $width  integer The raw requested width.
   * @param $height integer The raw requested height.
   * @return boolean
   */
  public function checkRequestedDimensions($width, $height) {
    if ($width != intval($width) || $height != intval($height)) {
      return FALSE;
    }

    // Only check of the image is not to large. It is possible for images to be
    // to small if their requested aspect ratio is way off, or if the settings
    // require strange aspect ratios.
    $drimage_config = $this->config('drimage.settings');
    if ($width > $drimage_config->get('downscale_x') || $height > $drimage_config->get('downscale_y')) {
      return FALSE;
    }

    // Check if the width is an exact match for the threshold value.
    if (($width - $drimage_config->get('upscale_x')) % $drimage_config->get('threshold') != 0) {
      return FALSE;
    }

    return TRUE;
  }

  /**
   * Try and find an image style that matches the requested dimensions and
   * drimage's settings.
   *
   * @param $requested_dimensions array The calculated requested dimensions.
   * @return mixed A matching image style or NULL if none was found.
   */
  public function findImageStyle($requested_dimensions) {
    // Get all styles and check if there is a usable one according to the
    // threshold settings. Get the image style that is closest to the requested
    // dimensions.
    $drimage_config = $this->config('drimage.settings');
    $threshold = $drimage_config->get('threshold');
    $focal_point = $this->moduleHandler()->moduleExists('focal_point');
    $styles = ImageStyle::loadMultiple();
    $image_style = NULL;
    // Use a million pixels difference to start with so basically any image
    // style will look like a better fit to drimage.
    $current_diff = 1000000;
    foreach ($styles as $name => $style) {
      if (stristr($name, 'drimage_')) {
        $translated_name = str_replace('drimage_', '', $name);
        if ($focal_point) {
          $translated_name = str_replace('focal_', '', $translated_name);
        }
        // If there is a height set, we will need a cropping image style. The
        // dimensions array will be keyed 0 = width and 1 = height. By default
        // we use the width to calculate difference in pixels, but for cropped
        // images we have to calculate the dimension to use. If the height is
        // bigger then the width, the height should be used.
        $dimensions = explode('_', $translated_name);
        $use_dimension = 0;
        if ($requested_dimensions[1] > $requested_dimensions[0]) {
          $use_dimension = 1;
        }
        $use_inverse_dimension = abs($use_dimension - 1);
        $calculated_diff = abs($dimensions[$use_dimension] - $requested_dimensions[$use_dimension]);
        if ($calculated_diff < $current_diff && $calculated_diff <= $threshold) {
          // When the height is given we can assume we need a cropping style.
          if ($requested_dimensions[1] > 0) {
            if ($dimensions[1] > 0) {
              // If using cropping we need to check if the ratio if the requested
              // image style and that of this image style won't distort our image
              // to much.
              $ratio_distortion = deg2rad($drimage_config->get('ratio_distortion') / 60);
              $ratio = $dimensions[$use_dimension] / $dimensions[$use_inverse_dimension];
              $requested_ratio = $requested_dimensions[$use_dimension] / $requested_dimensions[$use_inverse_dimension];
              if (abs(atan($ratio) - atan($requested_ratio)) <= $ratio_distortion) {
                $current_diff = $calculated_diff;
                $image_style = $styles[$name];
              }
            }
          }
          else {
            // This is just a simple scaling image style.
            if ($dimensions[1] == 0) {
              $current_diff = $calculated_diff;
              $image_style = $styles[$name];
            }
          }
        }
      }
    }

    // No usable image style could be found, so we will have to create one.
    if (empty($image_style)) {
      $image_style = $this->createDrimageStyle($requested_dimensions);
    }

    return $image_style;
  }

  /**
   * Create an image style from the requested dimensions.
   *
   * @param $requested_dimensions Array
   *    The array containing the dimensions.
   * @return mixed The image style or FALSE in case something went wrong.
   */
  public function createDrimageStyle($requested_dimensions) {
    $focal_point = $this->moduleHandler()->moduleExists('focal_point');

    $name = 'drimage_' . $requested_dimensions[0] . '_' . $requested_dimensions[1];
    $label = 'DrImage (' . $requested_dimensions[0] . 'x' . $requested_dimensions[1] . ')';

    // If focal point module is activated, name the styles accordingly.
    if ($focal_point) {
      $name = 'drimage_focal_' . $requested_dimensions[0] . '_' . $requested_dimensions[1];
      $label = 'DrImage Focal (' . $requested_dimensions[0] . 'x' . $requested_dimensions[1] . ')';
    }

    // When multiple images width the same dimension are requested in 1 page
    // we can sometimes trigger errors here. Image style can already be
    // created by another request that came in a few milliseconds before this
    // request. Catch that error and try and use the image style that was
    // already created.
    try {
      $style = ImageStyle::create(array('name' => $name, 'label' => $label));
      $configuration = array(
        'uuid' => NULL,
        'weight' => 0,
        'data' => array(
          'upscale' => FALSE,
          'width' => NULL,
          'height' => NULL,
        ),
      );
      $configuration['data']['width'] = $requested_dimensions[0];
      if ($requested_dimensions[1] > 0) {
        $configuration['data']['height'] = $requested_dimensions[1];
      }

      // Height is NULL by default, images are scaled.
      if ($configuration['data']['width'] == NULL || $configuration['data']['height'] == NULL) {
        $configuration['id'] = 'image_scale';
      }
      else {
        $configuration['id'] = 'image_scale_and_crop';

        // If focal point module is activated, use that image style instead.
        if ($focal_point) {
          $configuration['id'] = 'focal_point_scale_and_crop';
        }
      }

      $effect = \Drupal::service('plugin.manager.image.effect')->createInstance($configuration['id'], $configuration);
      $style->addImageEffect($effect->getConfiguration());
      $style->save();
      $styles[$name] = $style;
      $image_style = $styles[$name];
    }
    catch (EntityStorageException $e) {
      // Wait a tiny little bit (O.05s) to make sure another request isn't
      // still adding effects to the image style.
      usleep(50000);
      $image_style = ImageStyle::load($name);
    }
    catch (Exception $e) {
      return NULL;
    }

    return $image_style;
  }

  /**
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   * @param $width float
   *    The requested width in pixels that came from the JS.
   * @param $height float
   *    The requested height in pixels that came from the JS.
   * @param $fid integer
   *    The file id to render
   * @param $filename
   *    The filename, only here for SEO purposes.
   *
   * @return \Symfony\Component\HttpFoundation\BinaryFileResponse|\Symfony\Component\HttpFoundation\Response
   *   The transferred file as response or some error response.
   *
   * @throws \Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException
   *   Thrown when the user does not have access to the file.
   * @throws \Symfony\Component\HttpKernel\Exception\ServiceUnavailableHttpException
   *   Thrown when the file is still being generated.
   */
  public function image(Request $request, $width, $height, $fid, $filename) {
    // Bail out if the image is not valid.
    $file = File::load($fid);
    $image = $this->imageFactory->get($file->getFileUri());
    if (!$image->isValid()) {
      return new Response($this->t('Error generating image, invalid file.'), 500);
    }

    // Bail out if the arguments are not numbers.
    if (!is_numeric($width) || !is_numeric($height) || !is_numeric($fid)) {
      $error_msg = $this->t('Error generating image, invalid parameters.');
    }

    // The Javascript should have generated a nice size adhering to the
    // threshold and x/y up/down-scaling settings. Check if it actually did.
    // Return the fallback image if it didn't.
    if (!$this->checkRequestedDimensions($width, $height)) {
      $error_msg = $this->t('Error generating image, invalid dimensions.');
    }

    // Try and find a matching image style.
    $requested_dimensions = array(0 => $width, 1 => $height);
    $image_style = $this->findImageStyle($requested_dimensions);
    if (empty($image_style)) {
      $error_msg = $this->t('Could not find matching image style.');
    }

    // Variable translation to make the original imageStyle deliver method work.
    $image_uri = explode('://', $file->getFileUri());
    $scheme = $image_uri[0];
    $request->query->set('file', $image_uri[1]);

    if (!empty($error_msg)) {
      $drimage_config = $this->config('drimage.settings');
      $fallback_style = $drimage_config->get('fallback_style');
      if (!empty($fallback_style)) {
        $image_style = ImageStyle::load($fallback_style);
      }
    }

    if (!empty($image_style)) {
      // Because drimage does not use itok, we simulate it.
      if (!$this->config('image.settings')->get('allow_insecure_derivatives')) {
        $image_uri = $image_uri[0] . '://' . $image_uri[1];
        $request->query->set(IMAGE_DERIVATIVE_TOKEN, $image_style->getPathToken($image_uri));
      }

      return $this->deliver($request, $scheme, $image_style);
    }

    return New Response($error_msg, 500);
  }

}
