<?php

namespace Drupal\drimage\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\image\Entity\ImageStyle;

/**
 * Class DrimageSettingsForm
 * @package Drupal\drimage\Form
 */
class DrimageSettingsForm extends ConfigFormBase {

  /**
   * Returns a unique string identifying the form.
   *
   * @return string
   *   The unique string identifying the form.
   */
  public function getFormId() {
    return 'drimage_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'drimage.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // Provide some feedback on the insecure derivative implementation.
    if (!$this->config('image.settings')->get('allow_insecure_derivatives')) {
      drupal_set_message($this->t('The "allow_insecure_derivatives" settings is disabled, but drimage will bypass this.'), 'warning');
    }

    $form['threshold'] = array(
      '#type' => 'number',
      '#title' => t('Minimum difference per image style'),
      '#default_value' => $this->config('drimage.settings')->get('threshold'),
      '#description' => t('A minimum amount 2 image styles have to differ before a new image style is being created. For cropping styles, the biggest dimension is used. This feature will limit your disk space usage, but the quality of images might be less good since the browser has to scale them.'),
      '#min' => 1,
      '#max' => 500,
      '#step' => 1,
      '#field_suffix' => ' ' . t('pixels'),
    );

    $form['ratio_distortion'] = array(
      '#type' => 'number',
      '#title' => t('Maximum allowed ratio distortion'),
      '#default_value' => $this->config('drimage.settings')->get('ratio_distortion'),
      '#description' => t('How much ratio distortion is allowed when trying to reuse image styles that crop images. The aspect ratio of the generated images will be distorted by the browser to keep the exact aspect ratio your CSS rules require. A minimum of 30 minutes is required to allow for small rounding errors.'),
      '#min' => 1,
      '#max' => 3600,
      '#step' => 1,
      '#field_suffix' => ' ' . t('minutes. (1° = 60\')'),
    );

    $form['downscale_x'] = array(
      '#type' => 'number',
      '#title' => t('Maximum image style width'),
      '#default_value' => $this->config('drimage.settings')->get('downscale_x'),
      '#description' => t('The maximum width for the biggest image style. Anything bigger will be scaled down to this size unless aspect ratio\'s and other min/max settings force it otherwise.'),
      '#min' => 1,
      '#max' => 10000,
      '#step' => 1,
      '#field_suffix' => ' ' . t('pixels'),
    );

    $form['downscale_y'] = array(
      '#type' => 'number',
      '#title' => t('Maximum image style height'),
      '#default_value' => $this->config('drimage.settings')->get('downscale_y'),
      '#description' => t('The maximum height for the biggest image style. Anything bigger will be scaled down to this size unless aspect ratio\'s and other min/max settings force it otherwise. This only effects cropping images.'),
      '#min' => 1,
      '#max' => 10000,
      '#step' => 1,
      '#field_suffix' => ' ' . t('pixels'),
    );

    $form['upscale_x'] = array(
      '#type' => 'number',
      '#title' => t('Minimum image style width'),
      '#default_value' => $this->config('drimage.settings')->get('upscale_x'),
      '#description' => t('The minimal width for the smallest image style. Anything smaller will be scaled up to this size unless aspect ratio\'s and other min/max settings force it otherwise.'),
      '#min' => 1,
      '#max' => 500,
      '#step' => 1,
      '#field_suffix' => ' ' . t('pixels'),
    );

    $form['upscale_y'] = array(
      '#type' => 'number',
      '#title' => t('Minimum image style height'),
      '#default_value' => $this->config('drimage.settings')->get('upscale_y'),
      '#description' => t('The minimal height for the smallest image style. Anything smaller will be scaled up to this size unless aspect ratio\'s and other min/max settings force it otherwise. This only effects cropping images.'),
      '#min' => 1,
      '#max' => 500,
      '#step' => 1,
      '#field_suffix' => ' ' . t('pixels'),
    );

    $options = [];
    $styles = ImageStyle::loadMultiple();
    foreach ($styles as $style) {
      $id = $style->id();
      if (!stristr($id, 'drimage_')) {
        $options[$id] = $style->get('label');
      }
    }
    $form['fallback_style'] = array(
      '#type' => 'select',
      '#title' => t('Fallback Image Style'),
      '#default_value' => $this->config('drimage.settings')->get('fallback_style'),
      '#description' => t('If drimage cannot find an image style or the required dimensens are to small, fallback to using this image style instead of generating an error and not showing an image at all.'),
      '#empty_option' => t('None (original image)'),
      '#options' => $options,
    );

    $form['actions']['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Save'),
    );

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('drimage.settings')
      ->set('threshold', $form_state->getValue('threshold'))
      ->set('ratio_distortion', $form_state->getValue('ratio_distortion'))
      ->set('upscale_x', $form_state->getValue('upscale_x'))
      ->set('upscale_y', $form_state->getValue('upscale_y'))
      ->set('downscale_x', $form_state->getValue('downscale_x'))
      ->set('downscale_y', $form_state->getValue('downscale_y'))
      ->set('fallback_style', $form_state->getValue('fallback_style'))
      ->save();

    drupal_set_message(t('Drimage Settings have been successfully saved.'));
  }
}
